@@ -38,12 +38,12 @@ KSaneWidget::KSaneWidget(QWidget *parent)
     lay->addWidget(openDialog);
     setLayout(lay);
 
-    connect(openDialog, SIGNAL(clicked()), d, SLOT(ReOpenDialog()));
+    connect(openDialog, &QPushButton::clicked, d, &KSaneWidgetPrivate::ReOpenDialog);
 
     // Forward signals from the private class
     //connect(d, SIGNAL(scanProgress(int)), this, SIGNAL(scanProgress(int)));
-    connect(d, SIGNAL(ImageReady(QByteArray&,int,int,int,int)),
-            this, SIGNAL(imageReady(QByteArray&,int,int,int,int)));
+    connect(d, &KSaneWidgetPrivate::ImageReady, this, &KSaneWidget::imageReady);
+    connect(d, &KSaneWidgetPrivate::qImageReady, this, &KSaneWidget::scannedImageReady);
     //connect(d, SIGNAL(scanDone(int,QString)), this, SIGNAL(scanDone(int,QString)));
 
 }
@@ -65,7 +65,7 @@ bool KSaneWidget::openDevice(const QString &device_name)
     if (!d->OpenSource(device_name)) {
         return false;
     }
-    QTimer::singleShot(0, d, SLOT(OpenDialog()));
+    QTimer::singleShot(0, d, &KSaneWidgetPrivate::OpenDialog);
 
     return true;
 }
@@ -188,5 +188,4 @@ void KSaneWidget::setSelection(QPointF lefttop, QPointF rightbottom)
 void KSaneWidget::setOptionsCollapsed(bool) {}
 void KSaneWidget::setScanButtonHidden(bool) {}
 void KSaneWidget::initGetDeviceList() const {}
-
 }  // NameSpace KSaneIface
