@@ -1,9 +1,4 @@
 /* ============================================================
- *
- * This file is part of the KDE project
- *
- * Date        : 2007-09-13
- * Description : Sane interface for KDE
  *
  * SPDX-FileCopyrightText: 2007-2011 Kare Sars <kare.sars@iki .fi>
  *
@@ -43,6 +38,8 @@ public:
      * \param text is the text describing the checkbox.
      */
     LabeledCheckbox(QWidget *parent, const QString &text);
+
+    LabeledCheckbox(QWidget *parent, KSaneOption *option);
     ~LabeledCheckbox();
     void setChecked(bool);
     bool isChecked();
@@ -50,6 +47,12 @@ public:
 Q_SIGNALS:
     void toggled(bool);
 
+public Q_SLOTS:
+    void setValue(const QVariant &value);
+    
+protected:
+    void initCheckBox(const QString &name);
+    
 private:
 
     QCheckBox *chbx;
